﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;
using HeuristicLab.Common;
using HeuristicLab.DataAccess;
using HeuristicLab.PluginInfrastructure;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Hive.Contracts.BusinessObjects {

  [StorableClass]
  [DataContract]
  [Serializable]
  public class JobDto : PersistableObject {
    [Storable]
    [DataMember]
    public JobState State { get; set; }
    [Storable]
    [DataMember]
    public Guid? UserId { get; set; }
    [Storable]
    [DataMember]
    public SlaveDto Slave { get; set; }
    [Storable]
    [DataMember]
    public JobDto ParentJob { get; set; }
    [Storable]
    [DataMember]
    public double? Percentage { get; set; }
    [Storable]
    [DataMember]
    public String Exception { get; set; }
    [Storable]
    [DataMember]
    public DateTime? DateCreated { get; set; }
    [Storable]
    [DataMember]
    public DateTime? DateCalculated { get; set; }
    [Storable]
    [DataMember]
    public DateTime? DateFinished { get; set; }
    [Storable]
    [DataMember]
    public int Priority { get; set; }
    [Storable]
    [DataMember]
    public int CoresNeeded { get; set; }
    [Storable]
    [DataMember]
    public int MemoryNeeded { get; set; }
    [Storable]
    [DataMember]
    public List<HivePluginInfoDto> PluginsNeeded { get; set; }
    [Storable]
    [DataMember]
    public List<Guid> AssignedResourceIds { get; set; }
    [Storable]
    [DataMember]
    public ProjectDto Project { get; set; }

    public override string ToString() {
      return base.ToString() + "State: " + State + ", [Ressource : " + Slave + " ] , DateCreated: " + DateCreated + ", DateCalculated: " + DateCalculated +
        "Priority: " + Priority + ", CoresNeeded: " + CoresNeeded + "AssignedResources: " + AssignedResourceIds;
    }

    public JobDto() {
      PluginsNeeded = new List<HivePluginInfoDto>();
      AssignedResourceIds = new List<Guid>();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      JobDto clone = (JobDto)base.Clone(cloner);
      clone.Id = this.Id;
      clone.AssignedResourceIds = new List<Guid>(this.AssignedResourceIds);
      clone.Slave = (SlaveDto)cloner.Clone(this.Slave);
      clone.CoresNeeded = this.CoresNeeded;
      clone.DateCalculated = this.DateCalculated;
      clone.DateCreated = this.DateCreated;
      clone.DateFinished = this.DateFinished;
      clone.Exception = this.Exception;
      clone.Id = this.Id;
      clone.MemoryNeeded = this.MemoryNeeded;
      clone.ParentJob = (JobDto)cloner.Clone(this.ParentJob);
      clone.Percentage = this.Percentage;
      clone.PluginsNeeded = (from p in this.PluginsNeeded
                             select (HivePluginInfoDto)cloner.Clone(p)).ToList();
      clone.Priority = this.Priority;
      clone.Project = (ProjectDto)cloner.Clone(this.Project);
      clone.State = this.State;
      clone.UserId = this.UserId;
      return clone;
    }
  }
}
