﻿using System;
using System.ServiceModel;
using HeuristicLab.Common;

namespace HeuristicLab.Hive.Contracts {
  public class WcfServicePool<T> {
    private static object locker = new object();
    private string hostAddress;
    private string username;
    private string password;
    private string endpointName;
    private ChannelFactory<T> factory = null;
    private Disposable<T> disposableService = null;
    private int requestCount = 0;

    public WcfServicePool(string hostAddress, string username, string password, string endpointName) {
      this.hostAddress = hostAddress;
      this.username = username;
      this.password = password;
      this.endpointName = endpointName;
    }

    private T CreateFacade(string endpointName) {
      lock (locker) {
        try {
          return CreateChannel(endpointName);
        }
        catch (EndpointNotFoundException ex) {
          OnExceptionOccured(ex);
        }
        return default(T);
      }
    }

    protected virtual T CreateChannel(string endpointName) {
      if (factory == null) {
        factory = new ChannelFactory<T>(endpointName);
        WcfSettings.SetEndpointAddress(factory.Endpoint, hostAddress);

        factory.Credentials.UserName.UserName = username;
        factory.Credentials.UserName.Password = password;
      }

      return factory.CreateChannel();
    }

    public Disposable<T> GetService() {
      lock (locker) {
        requestCount++;
        if (disposableService == null) {
          disposableService = new Disposable<T>(CreateFacade(endpointName));
          disposableService.OnDisposing += new EventHandler(disposableService_OnDisposing);
        }
        return disposableService;
      }
    }

    private void disposableService_OnDisposing(object sender, EventArgs e) {
      DisposeService();
    }

    public void DisposeService() {
      lock (locker) {
        requestCount--;
        if (requestCount == 0) {
          WcfSettings.DisposeWcfClient((ICommunicationObject)disposableService.Obj);
          disposableService = null;
        } else if (requestCount < 0) {
          throw new WcfServicePoolException("requestCount cannot be less than 0.");
        }
      }
    }

    public event EventHandler<EventArgs<Exception>> ExceptionOccured;
    private void OnExceptionOccured(Exception e) {
      lock (locker) {
        var handler = ExceptionOccured;
        if (handler != null) handler(this, new EventArgs<Exception>(e));
      }
    }
  }
}
