﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.Hive.JobBase;
using HeuristicLab.Persistence.Default.Xml;
using System.IO;
using HeuristicLab.Hive.Experiment.Jobs;

namespace HeuristicLab.Hive.Experiment.Views {
  [View("JobConfigView")]
  [Content(typeof(IJob), true)]
  public partial class JobConfigView : ItemView {
    public new IJob Content {
      get { return (IJob)base.Content; }
      set {
        if (base.Content != value) {
          base.Content = value;
        }
      }
    }

    public JobConfigView() {
      InitializeComponent();
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content != null) {
        this.optimizerTypeTextBox.Text = Content.Name;
        this.memoryNeededTextBox.Text = Content.MemoryNeeded.ToString();
        this.coresNeededTextBox.Text = Content.CoresNeeded.ToString();
        this.computeInParallelCheckBox.Checked = Content.ComputeInParallel;
        this.jobConfigListView.Content = new JobList(Content.ChildJobs);
        SetEnabledStateOfControls();
      }
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
    }

    protected override void DeregisterContentEvents() {
      base.DeregisterContentEvents();
    }

    protected override void SetEnabledStateOfControls() {
      if (InvokeRequired) {
        this.InvokeIfRequired(c => { SetEnabledStateOfControls(); });
      } else {
        base.SetEnabledStateOfControls();
        this.optimizerTypeTextBox.ReadOnly = true;
        this.computeInParallelCheckBox.Enabled = this.ReadOnly;
        this.coresNeededTextBox.ReadOnly = this.ReadOnly;
        this.memoryNeededTextBox.ReadOnly = this.ReadOnly;

        if (Content != null) {
          if (Content.IsParallelizable) {
            this.computeInParallelCheckBox.Enabled = true;
          } else {
            this.computeInParallelCheckBox.Enabled = false;
            this.computeInParallelCheckBox.Checked = false;
          }
        }

        if (this.computeInParallelCheckBox.Checked) {
          if (!tabControl.TabPages.ContainsKey(childJobsTabPage.Name)) {
            tabControl.TabPages.Add(childJobsTabPage);
          }
        } else {
          if (tabControl.TabPages.ContainsKey(childJobsTabPage.Name)) {
            tabControl.TabPages.Remove(childJobsTabPage);
          }
        }
      }
    }

    private void UpdateContent(object sender, EventArgs e) {
      if (InvokeRequired)
        Invoke(new EventHandler(UpdateContent), sender, e);
      else {
        if (Content != null) {
          Content.CoresNeeded = int.Parse(this.coresNeededTextBox.Text);
          Content.MemoryNeeded = int.Parse(this.memoryNeededTextBox.Text);
          Content.ComputeInParallel = computeInParallelCheckBox.Checked;
        }
      }
    }
    
    private void coresNeededTextBox_Validated(object sender, EventArgs e) {
      UpdateContent(sender, e);
    }

    private void coresNeededTextBox_Validating(object sender, CancelEventArgs e) {
    }

    private void memoryNeededTextBox_Validated(object sender, EventArgs e) {
      UpdateContent(sender, e);
    }

    private void memoryNeededTextBox_Validating(object sender, CancelEventArgs e) {
    }

    void computeInParallelCheckBox_CheckedChanged(object sender, System.EventArgs e) {
      UpdateContent(sender, e);
      SetEnabledStateOfControls();
    }

  }
}
