﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Hive.Contracts.Interfaces;
using HeuristicLab.Hive.Contracts;
using HeuristicLab.Hive.Contracts.BusinessObjects;
using HeuristicLab.Hive.Server.DataAccess;
using HeuristicLab.Hive.Contracts.ResponseObjects;
using System.Security.Permissions;

namespace HeuristicLab.Hive.Server.Core {
  public class ClientFacade : IClientFacade {

    private IJobManager jobManager = ServiceLocator.GetJobManager();
    private IContextFactory contextFactory = ServiceLocator.GetContextFactory();

    public ClientFacade() {
    }

    #region IClientFacade Members

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Client)]
    public ResponseObject<JobDto> AddJob(SerializedJob job) {
      using (contextFactory.GetContext()) {
        return jobManager.AddNewJob(job);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Client)]
    public ResponseObject<JobDto> AddJobWithGroupStrings(SerializedJob job, IEnumerable<string> resources) {
      using (contextFactory.GetContext()) {
        return jobManager.AddJobWithGroupStrings(job, resources);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Client)]
    public Response RequestSnapshot(Guid jobId) {
      using (contextFactory.GetContext()) {
        ServiceLocator.GetAuthorizationManager().AuthorizeForJobs(jobId);
        return jobManager.RequestSnapshot(jobId);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Client)]
    public ResponseObject<SerializedJob> GetLastSerializedResult(Guid jobId) {
      using (contextFactory.GetContext(false)) {
        ServiceLocator.GetAuthorizationManager().AuthorizeForJobs(jobId);
        return jobManager.GetLastSerializedResult(jobId);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Client)]
    public ResponseObject<SerializedJob> GetSnapshotResult(Guid jobId) {
      using (contextFactory.GetContext(false)) {
        ServiceLocator.GetAuthorizationManager().AuthorizeForJobs(jobId);
        return jobManager.GetSnapshotResult(jobId);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Client)]
    public ResponseObject<JobDto> GetJobById(Guid jobId) {
      using (contextFactory.GetContext(false)) {
        ServiceLocator.GetAuthorizationManager().AuthorizeForJobs(jobId);
        return jobManager.GetJobById(jobId);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Client)]
    public Response AbortJob(Guid jobId) {
      using (contextFactory.GetContext()) {
        ServiceLocator.GetAuthorizationManager().AuthorizeForJobs(jobId);
        return jobManager.AbortJob(jobId);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Client)]
    public ResponseObject<JobResultList> GetJobResults(IEnumerable<Guid> jobIds) {
      using (contextFactory.GetContext(false)) {
        ServiceLocator.GetAuthorizationManager().AuthorizeForJobs(jobIds.ToArray());
        return jobManager.GetJobResults(jobIds);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Client)]
    public ResponseObject<JobResultList> GetChildJobResults(Guid? parentJobId, bool recursive, bool includeParent) {
      using (contextFactory.GetContext(false)) {
        if (parentJobId.HasValue) {
          ServiceLocator.GetAuthorizationManager().AuthorizeForJobs(parentJobId.Value);
        } 
        // If parentJobId is null, GetChildJobResults will filter the results for the current user
        return jobManager.GetChildJobResults(parentJobId, recursive, includeParent);
      }
    }
    #endregion


  }
}
