﻿using System;
using System.Collections.Generic;
using System.Text;
using HeuristicLab.Hive.Contracts.BusinessObjects;
using System.IO;

namespace HeuristicLab.Hive.Server.DataAccess {
  public interface IJobDao: IGenericDao<JobDto> {
    IEnumerable<JobDto> FindActiveJobsOfSlave(SlaveDto slave);

    IEnumerable<JobDto> GetJobsByState(JobState state);

    void AssignSlaveToJob(Guid slaveId, Guid jobId);

    void UnAssignSlaveToJob(Guid jobId);

    void SetJobOffline(JobDto job);

    SerializedJob InsertWithAttachedJob(SerializedJob job);

    byte[] GetBinaryJobFile(Guid jobId); 

    IEnumerable<JobDto> FindFittingJobs(JobState state, int freeCores, int freeMemory, Guid slaveGuid);

    Stream GetSerializedJobStream(Guid jobId);

    IEnumerable<JobDto> FindWithLimitations(JobState jobState, int offset, int count);

    void SetBinaryJobFile(Guid jobId, byte[] data);

    IEnumerable<JobDto> FindJobsById(IEnumerable<Guid> jobIds);

    bool IsUserAuthorizedForJobs(Guid userId, params Guid[] jobIds);

    IEnumerable<JobDto> FindJobsByParentId(Guid? parentJobId, bool recursive);

    /// <summary>
    /// Returns a list of jobs which are on state WaitForChildJobs and those child-jobs are finished
    /// </summary>
    IEnumerable<JobDto> FindJobsWithFinishedChilds(Guid guid);
  }
}
