﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Hive.Contracts.BusinessObjects;
using HeuristicLab.Hive.Server.DataAccess;
using System.Threading;

namespace HeuristicLab.Hive.Server.LINQDataAccess {
  public class SlaveDao: BaseDao<SlaveDto, Slave>, ISlaveDao {

    public SlaveDao() {      
    }

    public SlaveDto FindById(Guid id) {
      return (from slave in Context.Resources.OfType<Slave>()                         
              where slave.ResourceId.Equals(id)
              select EntityToDto(slave, null)
            ).SingleOrDefault();      
    }

    public IEnumerable<SlaveDto> FindAll() {
      return (from slave in Context.Resources.OfType<Slave>()
              select
                EntityToDto(slave, null)
             ).ToList();
    }

    public IEnumerable<SlaveDto> FindAllSlavesWithoutGroup() {
      return (from slave in Context.Resources.OfType<Slave>()
              where slave.SlaveGroup_Resources_Parents.Count == 0
              select EntityToDto(slave, null)).ToList();
    }

    public SlaveDto GetSlaveForJob(Guid jobId) {
      return (from job in Context.Jobs
              where job.JobId.Equals(jobId)
              select EntityToDto(job.Slave, null)).SingleOrDefault();
    }

    public void SetServerSideCalendar(SlaveDto slave, Guid slaveGroupId) {
      Slave dbslave = Context.Resources.OfType<Slave>().SingleOrDefault(c => c.ResourceId.Equals(slave.Id));
      dbslave.UseCalendarFromResourceId = slaveGroupId;
      dbslave.CalendarSyncStatus = Enum.GetName(typeof(CalendarState), CalendarState.Fetch);
      CommitChanges();
    }

    public SlaveDto Insert(SlaveDto info) {
      Slave c = DtoToEntity(info, null);      
      Context.Resources.InsertOnSubmit(c);
      CommitChanges();
      info.Id = c.ResourceId;
      return info;
    }

    //Cascading delete takes care of the rest
    public void Delete(SlaveDto info) {
      Resource res = Context.Resources.SingleOrDefault(c => c.ResourceId.Equals(info.Id));            
      Context.Resources.DeleteOnSubmit(res);
      CommitChanges();
    }

    public void Update(SlaveDto info) {
      Slave slave = Context.Resources.OfType<Slave>().SingleOrDefault(c => c.ResourceId.Equals(info.Id));
      DtoToEntity(info, slave);
      CommitChanges();      
    }

    public override Slave DtoToEntity(SlaveDto source, Slave target) {
      if (source == null)
        return null;
      if (target == null) 
        target = new Slave();
      
      target.CPUSpeed = source.CpuSpeedPerCore;
      
      target.FreeMemory = source.FreeMemory;
      target.Name = source.Name;
      target.ResourceId = source.Id;
      target.CalendarSyncStatus = Enum.GetName(typeof(CalendarState), source.CalendarSyncStatus);
      target.Login = source.Login;
      target.Memory = source.Memory;
      target.NumberOfCores = source.NrOfCores;
      target.NumberOfFreeCores = source.NrOfFreeCores;
      target.Status = source.State.ToString();
      return target;
    }

    public override SlaveDto EntityToDto(Slave source, SlaveDto target) {
      if (source == null)
        return null;
      if(target == null) 
        target = new SlaveDto();
      target.CpuSpeedPerCore = source.CPUSpeed;
      target.FreeMemory = source.FreeMemory ?? 0;
      target.Id = source.ResourceId;
      target.CalendarSyncStatus = (CalendarState) Enum.Parse(typeof (CalendarState), source.CalendarSyncStatus);
      target.Login = source.Login;
      target.Memory = source.Memory ?? 0;
      target.Name = source.Name;
      target.NrOfCores = source.NumberOfCores ?? 0;
      target.NrOfFreeCores = source.NumberOfFreeCores ?? 0;
      target.State = (SlaveState)Enum.Parse(typeof(SlaveState), source.Status, true);
      target.IsAllowedToCalculate = source.IsAllowedToCalculate;
      return target;
    } 
  }
}
